from os.path import getsize
from struct import unpack


class Resource(object):
    def __init__(self):
        super(Resource, self).__init__()
        self.file_path = None
        self.file = None
        self.chunks = [''] * 49

    def open(self, file_path):
        self.file_path = file_path
        self.file = open(self.file_path, 'rb')


class Polygon(object):
    def __init__(self):
        self.resource = None
        self.offset = None
        self.data = ''

    def read(self, resource, offset, data):
        self.resource = resource
        self.offset = offset
        self.data = data


class Resources(object):
    def __init__(self):
        super(Resources, self).__init__()
        self.toc = {}
        self.chunks = [None] * 49

    def read(self, files):
        for file_path in files:
            resource = Resource()
            resource.open(file_path)
            toc = list(unpack('<49I', resource.file.read(0xc4)))
            resource.file.seek(0)
            data = resource.file.read()
            resource.file.close()
            toc.append(getsize(file_path))
            for i, entry in enumerate(toc[:-1]):
                if self.chunks[i] is not None:
                    continue
                begin = toc[i]
                if begin == 0:
                    continue
                end = None
                for j in range(i + 1, len(toc)):
                    if toc[j]:
                        end = toc[j]
                        break
                resource.chunks[i] = data[begin:end]
                print i, resource.file_path, begin, end
                self.chunks[i] = resource
            self.toc = toc

    def get_tex_3gon_xyz(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8
        for i in range(tri_count):
            polygon_data = data[offset:offset+18]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 18

    def get_tex_4gon_xyz(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18
        for i in range(quad_count):
            polygon_data = data[offset:offset+24]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 24

    def get_untex_3gon_xyz(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18 + quad_count * 24
        for i in range(untri_count):
            polygon_data = data[offset:offset+18]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 18

    def get_untex_4gon_xyz(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18 + quad_count * 24 + untri_count * 18
        for i in range(unquad_count):
            polygon_data = data[offset:offset+24]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 24

    def get_tex_3gon_norm(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18 + quad_count * 24 + untri_count * 18 + unquad_count * 24
        for i in range(tri_count):
            polygon_data = data[offset:offset+18]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 18

    def get_tex_4gon_norm(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18 + quad_count * 24 + untri_count * 18 + unquad_count * 24 + tri_count * 18
        for i in range(quad_count):
            polygon_data = data[offset:offset+24]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 24

    def get_tex_3gon_uv(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18 + quad_count * 24 + untri_count * 18 + unquad_count * 24 + tri_count * 18 + quad_count * 24
        for i in range(tri_count):
            polygon_data = data[offset:offset+10]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 10

    def get_tex_4gon_uv(self, toc_offset=0x40):
        toc_pointer = self.toc[toc_offset / 4]
        resource = self.chunks[toc_offset / 4]
        data = resource.chunks[toc_offset / 4]
        (tri_count, quad_count, untri_count, unquad_count) = unpack('<4H', data[0:8])
        offset = 8 + tri_count * 18 + quad_count * 24 + untri_count * 18 + unquad_count * 24 + tri_count * 18 + quad_count * 24 + tri_count * 10
        for i in range(quad_count):
            polygon_data = data[offset:offset+12]
            polygon = Polygon()
            polygon.read(resource, toc_pointer + offset, polygon_data)
            yield polygon
            offset += 12

    def get_dir_light_rgb(self):
        data = self.chunks[0x64 / 4].chunks[0x64 / 4]
        begin = 0
        for i in range(3):
            yield data[begin:begin+2] + data[begin+6:begin+8] + data[begin+12:begin+14]
            begin += 2

    def get_dir_light_norm(self):
        data = self.chunks[0x64 / 4].chunks[0x64 / 4]
        begin = 18
        for i in range(3):
            yield data[begin:begin+6]
            begin += 6

    def get_amb_light_rgb(self):
        data = self.chunks[0x64 / 4].chunks[0x64 / 4]
        begin = 36
        return data[begin:begin+3]

    def get_background(self):
        data = self.chunks[0x64 / 4].chunks[0x64 / 4]
        begin = 39
        return data[begin:begin+6]

    def get_terrain_dims(self):
        data = self.chunks[0x68 / 4].chunks[0x68 / 4]
        begin = 0
        return data[begin:begin+2]
