# Initialize window:
from pandac.PandaModules import WindowProperties
wp = WindowProperties()
wp.setTitle('Ganesha')
import direct.directbase.DirectStart
base.win.requestProperties(wp)
# Normal imports:
from pandac.PandaModules import CullFaceAttrib
from direct.showbase.DirectObject import DirectObject
from direct.fsm.FSM import FSM
import wx
from world import World


def tell_source(polygon):
    class_name = polygon.source.__class__.__name__
    point_source = polygon.source.point_source
    if polygon.source.A.normal:
        normal_source = polygon.source.normal_source
        texcoord_source = polygon.source.texcoord_source
    print """Data for %s found in file:""" % (class_name,)
    print """%s""" % (point_source[0],)
    print """%8s  Data""" % ('Address',)
    print """%8u  XYZ coordinates""" % (point_source[1],)
    if polygon.source.A.normal:
        print """%8u  Normal vectors""" % (normal_source[1],)
        print """%8u  Texture coordinates""" % (texcoord_source[1],)
    print


class Mouse(DirectObject):
    def __init__(self, app):
        self.app = app
        self.init_collide()
        self.has_mouse = None
        self.prev_pos = None
        self.pos = None
        self.drag_start = None
        self.hovered_object = None
        self.button2 = False
        self.mouseTask = taskMgr.add(self.mouse_task, 'mouseTask')
        self.task = None
        self.accept('mouse1', self.mouse1)
        self.accept('mouse1-up', self.mouse1_up)
        self.accept('mouse2', self.rotateCamera)
        self.accept('mouse2-up', self.stopCamera)
        self.accept('wheel_up', self.zoomIn)
        self.accept('wheel_down', self.zoomOut)

    def init_collide(self):
        from pandac.PandaModules import CollisionTraverser, CollisionNode, GeomNode
        from pandac.PandaModules import CollisionHandlerQueue, CollisionRay
        self.cTrav = CollisionTraverser('MousePointer')
        self.cQueue = CollisionHandlerQueue()
        self.cNode = CollisionNode('MousePointer')
        self.cNodePath = base.camera.attachNewNode(self.cNode)
        self.cNode.setFromCollideMask(GeomNode.getDefaultCollideMask())
        self.cRay = CollisionRay()
        self.cNode.addSolid(self.cRay)
        self.cTrav.addCollider(self.cNodePath, self.cQueue)

    def find_object(self):
        if self.app.world.node_path:
            self.cRay.setFromLens(base.camNode, self.pos.getX(), self.pos.getY())
            self.cTrav.traverse(self.app.world.node_path)
            if self.cQueue.getNumEntries() > 0:
                self.cQueue.sortEntries()
                return self.cQueue.getEntry(0).getIntoNodePath()
        return None

    def mouse_task(self, task):
        from pandac.PandaModules import Point2
        action = task.cont
        self.has_mouse = base.mouseWatcherNode.hasMouse()
        if self.has_mouse:
            self.pos = base.mouseWatcherNode.getMouse()
            if self.prev_pos:
                self.delta = self.pos - self.prev_pos
            else:
                self.delta = None
            if self.task:
                action = self.task(task)
        else:
            self.pos = None
        if self.pos:
            self.prev_pos = Point2(self.pos.getX(), self.pos.getY())
        return action

    def hover(self, task):
        if self.hovered_object:
            self.hovered_object.unhover()
            self.hovered_object = None
        if self.button2:
            self.camera_drag()
        hovered_node_path = self.find_object()
        if hovered_node_path:
            tile = hovered_node_path.findNetTag('polygon_i')
            if not tile.isEmpty():
                tag = tile.getTag('polygon_i')
                i = int(tag)
                self.hovered_object = self.app.world.polygons[i]
                self.hovered_object.hover()
        return task.cont

    def mouse1(self):
        self.app.state.request('mouse1')

    def mouse1_up(self):
        self.app.state.request('mouse1-up')

    def camera_drag(self):
        if self.delta:
            old_heading = base.camera.getH()
            new_heading = old_heading - self.delta.getX() * 180
            new_heading = new_heading % 360
            old_pitch = base.camera.getP()
            new_pitch = old_pitch + self.delta.getY() * 90
            new_pitch = max(-90, min(0, new_pitch))
            new_heading = 270 + new_heading
            new_pitch = - new_pitch
            self.app.world.set_camera_angle(new_heading, new_pitch)

    def rotateCamera(self):
        self.button2 = True
        self.app.state.request('mouse2')

    def stopCamera(self):
        self.button2 = False
        self.app.state.request('mouse2-up')

    def zoomIn(self):
        lens = base.cam.node().getLens()
        size = lens.getFilmSize()
        lens.setFilmSize(size / 1.2)
        scale = self.app.world.background.node_path.getScale()
        self.app.world.background.node_path.setScale(scale / 1.2)

    def zoomOut(self):
        lens = base.cam.node().getLens()
        size = lens.getFilmSize()
        lens.setFilmSize(size * 1.2)
        scale = self.app.world.background.node_path.getScale()
        self.app.world.background.node_path.setScale(scale *1.2)


class ViewerState(FSM):
    def __init__(self, app, name):
        FSM.__init__(self, name)
        self.app = app

    def enterSpin(self):
        #print 'enterSpin'
        taskMgr.add(self.app.world.spin_camera, 'spin_camera')
        self.app.mouse.task = self.app.mouse.hover

    def filterSpin(self, request, args):
        #print 'filterSpin'
        if request == 'mouse2':
            return 'FreeRotate'
        if request == 'mouse1':
            clicked = self.app.mouse.hovered_object
            if clicked:
                tell_source(clicked)
        return None

    def exitSpin(self):
        #print 'exitSpin'
        taskMgr.remove('spin_camera')
        self.app.mouse.task = None

    def enterFreeRotate(self):
        #print 'enterFreeRotate'
        self.app.mouse.task = self.app.mouse.hover

    def filterFreeRotate(self, request, args):
        if request == 'mouse1':
            clicked = self.app.mouse.hovered_object
            if clicked:
                tell_source(clicked)
        return None

    def exitFreeRotate(self):
        #print 'exitFreeRotate'
        self.app.mouse.task = None


class ViewerMouse(Mouse):
    pass


class Map_Viewer(DirectObject):
    def __init__(self, *args, **kwargs):
        self.state = ViewerState(self, 'viewer_state')
        self.mouse = ViewerMouse(self)
        self.world = World(self)
        self.accept(']', self.world.next_situation)
        self.accept('[', self.world.prev_situation)
        self.accept('n', self.world.next_gns)
        base.disableMouse()
        self.state.request('Spin')

    def handle_wx_events(self, task):
        while self.wxapp.Pending():
            self.wxapp.Dispatch()
        return task.cont

    def file_dialog(self):
        dlg = wx.FileDialog(self.wxwin, "Choose a GNS file", wildcard = 'GNS Files (*.GNS)|*.GNS;*.gns')
        if dlg.ShowModal() == wx.ID_OK:
            return dlg.GetPath()
        dlg.Destroy()
        return None

    def start(self, gns_path):
        self.wxapp = wx.App(0)
        self.wxwin = wx.Frame(None, -1, 'Ganesha wxWindow', wx.DefaultPosition)
        taskMgr.add(self.handle_wx_events, 'handle_wx_events')
        render.setAttrib(CullFaceAttrib.make(CullFaceAttrib.MCullCounterClockwise))
        self.world.read_gns(gns_path)
        self.world.read()
        run()
